/****************************************************************************
** Meta object code from reading C++ file 'searchview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/searchview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'searchview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SearchView_t {
    QByteArrayData data[21];
    char stringdata0[236];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SearchView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SearchView_t qt_meta_stringdata_SearchView = {
    {
QT_MOC_LITERAL(0, 0, 10), // "SearchView"
QT_MOC_LITERAL(1, 11, 6), // "search"
QT_MOC_LITERAL(2, 18, 0), // ""
QT_MOC_LITERAL(3, 19, 13), // "SearchParams*"
QT_MOC_LITERAL(4, 33, 6), // "appear"
QT_MOC_LITERAL(5, 40, 9), // "disappear"
QT_MOC_LITERAL(6, 50, 5), // "watch"
QT_MOC_LITERAL(7, 56, 5), // "query"
QT_MOC_LITERAL(8, 62, 12), // "watchChannel"
QT_MOC_LITERAL(9, 75, 9), // "channelId"
QT_MOC_LITERAL(10, 85, 13), // "watchKeywords"
QT_MOC_LITERAL(11, 99, 11), // "textChanged"
QT_MOC_LITERAL(12, 111, 4), // "text"
QT_MOC_LITERAL(13, 116, 17), // "searchTypeChanged"
QT_MOC_LITERAL(14, 134, 5), // "index"
QT_MOC_LITERAL(15, 140, 18), // "suggestionAccepted"
QT_MOC_LITERAL(16, 159, 11), // "Suggestion*"
QT_MOC_LITERAL(17, 171, 10), // "suggestion"
QT_MOC_LITERAL(18, 182, 20), // "onChannelSuggestions"
QT_MOC_LITERAL(19, 203, 20), // "QVector<Suggestion*>"
QT_MOC_LITERAL(20, 224, 11) // "suggestions"

    },
    "SearchView\0search\0\0SearchParams*\0"
    "appear\0disappear\0watch\0query\0watchChannel\0"
    "channelId\0watchKeywords\0textChanged\0"
    "text\0searchTypeChanged\0index\0"
    "suggestionAccepted\0Suggestion*\0"
    "suggestion\0onChannelSuggestions\0"
    "QVector<Suggestion*>\0suggestions"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SearchView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   72,    2, 0x0a /* Public */,
       5,    0,   73,    2, 0x0a /* Public */,
       6,    1,   74,    2, 0x0a /* Public */,
       8,    1,   77,    2, 0x0a /* Public */,
      10,    1,   80,    2, 0x0a /* Public */,
       6,    0,   83,    2, 0x08 /* Private */,
      11,    1,   84,    2, 0x08 /* Private */,
      13,    1,   87,    2, 0x08 /* Private */,
      15,    1,   90,    2, 0x08 /* Private */,
      18,    1,   93,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, 0x80000000 | 19,   20,

       0        // eod
};

void SearchView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SearchView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->search((*reinterpret_cast< SearchParams*(*)>(_a[1]))); break;
        case 1: _t->appear(); break;
        case 2: _t->disappear(); break;
        case 3: _t->watch((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->watchChannel((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->watchKeywords((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->watch(); break;
        case 7: _t->textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->searchTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->suggestionAccepted((*reinterpret_cast< Suggestion*(*)>(_a[1]))); break;
        case 10: _t->onChannelSuggestions((*reinterpret_cast< const QVector<Suggestion*>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SearchView::*)(SearchParams * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SearchView::search)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SearchView::staticMetaObject = { {
    QMetaObject::SuperData::link<View::staticMetaObject>(),
    qt_meta_stringdata_SearchView.data,
    qt_meta_data_SearchView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SearchView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SearchView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SearchView.stringdata0))
        return static_cast<void*>(this);
    return View::qt_metacast(_clname);
}

int SearchView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = View::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void SearchView::search(SearchParams * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
