/****************************************************************************
** Meta object code from reading C++ file 'playlistmodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/playlistmodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PlaylistModel_t {
    QByteArrayData data[28];
    char stringdata0[346];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PlaylistModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PlaylistModel_t qt_meta_stringdata_PlaylistModel = {
    {
QT_MOC_LITERAL(0, 0, 13), // "PlaylistModel"
QT_MOC_LITERAL(1, 14, 18), // "activeVideoChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 6), // "Video*"
QT_MOC_LITERAL(4, 41, 5), // "video"
QT_MOC_LITERAL(5, 47, 13), // "previousVideo"
QT_MOC_LITERAL(6, 61, 16), // "needSelectionFor"
QT_MOC_LITERAL(7, 78, 15), // "QVector<Video*>"
QT_MOC_LITERAL(8, 94, 6), // "videos"
QT_MOC_LITERAL(9, 101, 15), // "haveSuggestions"
QT_MOC_LITERAL(10, 117, 11), // "suggestions"
QT_MOC_LITERAL(11, 129, 10), // "searchMore"
QT_MOC_LITERAL(12, 140, 12), // "searchNeeded"
QT_MOC_LITERAL(13, 153, 9), // "addVideos"
QT_MOC_LITERAL(14, 163, 9), // "newVideos"
QT_MOC_LITERAL(15, 173, 14), // "searchFinished"
QT_MOC_LITERAL(16, 188, 5), // "total"
QT_MOC_LITERAL(17, 194, 11), // "searchError"
QT_MOC_LITERAL(18, 206, 7), // "message"
QT_MOC_LITERAL(19, 214, 15), // "emitDataChanged"
QT_MOC_LITERAL(20, 230, 13), // "setHoveredRow"
QT_MOC_LITERAL(21, 244, 3), // "row"
QT_MOC_LITERAL(22, 248, 10), // "clearHover"
QT_MOC_LITERAL(23, 259, 16), // "updateHoveredRow"
QT_MOC_LITERAL(24, 276, 16), // "enterAuthorHover"
QT_MOC_LITERAL(25, 293, 15), // "exitAuthorHover"
QT_MOC_LITERAL(26, 309, 18), // "enterAuthorPressed"
QT_MOC_LITERAL(27, 328, 17) // "exitAuthorPressed"

    },
    "PlaylistModel\0activeVideoChanged\0\0"
    "Video*\0video\0previousVideo\0needSelectionFor\0"
    "QVector<Video*>\0videos\0haveSuggestions\0"
    "suggestions\0searchMore\0searchNeeded\0"
    "addVideos\0newVideos\0searchFinished\0"
    "total\0searchError\0message\0emitDataChanged\0"
    "setHoveredRow\0row\0clearHover\0"
    "updateHoveredRow\0enterAuthorHover\0"
    "exitAuthorHover\0enterAuthorPressed\0"
    "exitAuthorPressed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PlaylistModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   94,    2, 0x06 /* Public */,
       6,    1,   99,    2, 0x06 /* Public */,
       9,    1,  102,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,  105,    2, 0x0a /* Public */,
      12,    0,  106,    2, 0x0a /* Public */,
      13,    1,  107,    2, 0x0a /* Public */,
      15,    1,  110,    2, 0x0a /* Public */,
      17,    1,  113,    2, 0x0a /* Public */,
      19,    0,  116,    2, 0x0a /* Public */,
      20,    1,  117,    2, 0x0a /* Public */,
      22,    0,  120,    2, 0x0a /* Public */,
      23,    0,  121,    2, 0x0a /* Public */,
      24,    0,  122,    2, 0x0a /* Public */,
      25,    0,  123,    2, 0x0a /* Public */,
      26,    0,  124,    2, 0x0a /* Public */,
      27,    0,  125,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    4,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::QStringList,   10,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,   14,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::QString,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PlaylistModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PlaylistModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activeVideoChanged((*reinterpret_cast< Video*(*)>(_a[1])),(*reinterpret_cast< Video*(*)>(_a[2]))); break;
        case 1: _t->needSelectionFor((*reinterpret_cast< const QVector<Video*>(*)>(_a[1]))); break;
        case 2: _t->haveSuggestions((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 3: _t->searchMore(); break;
        case 4: _t->searchNeeded(); break;
        case 5: _t->addVideos((*reinterpret_cast< const QVector<Video*>(*)>(_a[1]))); break;
        case 6: _t->searchFinished((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->searchError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->emitDataChanged(); break;
        case 9: _t->setHoveredRow((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->clearHover(); break;
        case 11: _t->updateHoveredRow(); break;
        case 12: _t->enterAuthorHover(); break;
        case 13: _t->exitAuthorHover(); break;
        case 14: _t->enterAuthorPressed(); break;
        case 15: _t->exitAuthorPressed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PlaylistModel::*)(Video * , Video * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlaylistModel::activeVideoChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PlaylistModel::*)(const QVector<Video*> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlaylistModel::needSelectionFor)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PlaylistModel::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlaylistModel::haveSuggestions)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PlaylistModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_PlaylistModel.data,
    qt_meta_data_PlaylistModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PlaylistModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PlaylistModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int PlaylistModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void PlaylistModel::activeVideoChanged(Video * _t1, Video * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PlaylistModel::needSelectionFor(const QVector<Video*> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PlaylistModel::haveSuggestions(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
