/****************************************************************************
** Meta object code from reading C++ file 'channelview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/channelview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'channelview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ChannelView_t {
    QByteArrayData data[27];
    char stringdata0[324];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ChannelView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ChannelView_t qt_meta_stringdata_ChannelView = {
    {
QT_MOC_LITERAL(0, 0, 11), // "ChannelView"
QT_MOC_LITERAL(1, 12, 9), // "activated"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 12), // "VideoSource*"
QT_MOC_LITERAL(4, 36, 11), // "videoSource"
QT_MOC_LITERAL(5, 48, 6), // "appear"
QT_MOC_LITERAL(6, 55, 9), // "disappear"
QT_MOC_LITERAL(7, 65, 13), // "itemActivated"
QT_MOC_LITERAL(8, 79, 11), // "QModelIndex"
QT_MOC_LITERAL(9, 91, 5), // "index"
QT_MOC_LITERAL(10, 97, 15), // "showContextMenu"
QT_MOC_LITERAL(11, 113, 5), // "point"
QT_MOC_LITERAL(12, 119, 17), // "toggleShowUpdated"
QT_MOC_LITERAL(13, 137, 6), // "enable"
QT_MOC_LITERAL(14, 144, 9), // "setSortBy"
QT_MOC_LITERAL(15, 154, 6), // "SortBy"
QT_MOC_LITERAL(16, 161, 6), // "sortBy"
QT_MOC_LITERAL(17, 168, 13), // "setSortByName"
QT_MOC_LITERAL(18, 182, 16), // "setSortByUpdated"
QT_MOC_LITERAL(19, 199, 14), // "setSortByAdded"
QT_MOC_LITERAL(20, 214, 20), // "setSortByLastWatched"
QT_MOC_LITERAL(21, 235, 20), // "setSortByMostWatched"
QT_MOC_LITERAL(22, 256, 16), // "markAllAsWatched"
QT_MOC_LITERAL(23, 273, 21), // "unwatchedCountChanged"
QT_MOC_LITERAL(24, 295, 5), // "count"
QT_MOC_LITERAL(25, 301, 11), // "updateQuery"
QT_MOC_LITERAL(26, 313, 10) // "transition"

    },
    "ChannelView\0activated\0\0VideoSource*\0"
    "videoSource\0appear\0disappear\0itemActivated\0"
    "QModelIndex\0index\0showContextMenu\0"
    "point\0toggleShowUpdated\0enable\0setSortBy\0"
    "SortBy\0sortBy\0setSortByName\0"
    "setSortByUpdated\0setSortByAdded\0"
    "setSortByLastWatched\0setSortByMostWatched\0"
    "markAllAsWatched\0unwatchedCountChanged\0"
    "count\0updateQuery\0transition"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ChannelView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   94,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   97,    2, 0x0a /* Public */,
       6,    0,   98,    2, 0x0a /* Public */,
       7,    1,   99,    2, 0x08 /* Private */,
      10,    1,  102,    2, 0x08 /* Private */,
      12,    1,  105,    2, 0x08 /* Private */,
      14,    1,  108,    2, 0x08 /* Private */,
      17,    0,  111,    2, 0x08 /* Private */,
      18,    0,  112,    2, 0x08 /* Private */,
      19,    0,  113,    2, 0x08 /* Private */,
      20,    0,  114,    2, 0x08 /* Private */,
      21,    0,  115,    2, 0x08 /* Private */,
      22,    0,  116,    2, 0x08 /* Private */,
      23,    1,  117,    2, 0x08 /* Private */,
      25,    1,  120,    2, 0x08 /* Private */,
      25,    0,  123,    2, 0x28 /* Private | MethodCloned */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::QPoint,   11,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void,

       0        // eod
};

void ChannelView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ChannelView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activated((*reinterpret_cast< VideoSource*(*)>(_a[1]))); break;
        case 1: _t->appear(); break;
        case 2: _t->disappear(); break;
        case 3: _t->itemActivated((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 4: _t->showContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 5: _t->toggleShowUpdated((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->setSortBy((*reinterpret_cast< SortBy(*)>(_a[1]))); break;
        case 7: _t->setSortByName(); break;
        case 8: _t->setSortByUpdated(); break;
        case 9: _t->setSortByAdded(); break;
        case 10: _t->setSortByLastWatched(); break;
        case 11: _t->setSortByMostWatched(); break;
        case 12: _t->markAllAsWatched(); break;
        case 13: _t->unwatchedCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->updateQuery((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->updateQuery(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ChannelView::*)(VideoSource * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChannelView::activated)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ChannelView::staticMetaObject = { {
    QMetaObject::SuperData::link<View::staticMetaObject>(),
    qt_meta_stringdata_ChannelView.data,
    qt_meta_data_ChannelView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ChannelView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ChannelView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ChannelView.stringdata0))
        return static_cast<void*>(this);
    return View::qt_metacast(_clname);
}

int ChannelView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = View::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void ChannelView::activated(VideoSource * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
