/****************************************************************************
** Meta object code from reading C++ file 'mediaview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/mediaview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mediaview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MediaView_t {
    QByteArrayData data[74];
    char stringdata0[939];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MediaView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MediaView_t qt_meta_stringdata_MediaView = {
    {
QT_MOC_LITERAL(0, 0, 9), // "MediaView"
QT_MOC_LITERAL(1, 10, 6), // "search"
QT_MOC_LITERAL(2, 17, 0), // ""
QT_MOC_LITERAL(3, 18, 13), // "SearchParams*"
QT_MOC_LITERAL(4, 32, 12), // "searchParams"
QT_MOC_LITERAL(5, 45, 14), // "setVideoSource"
QT_MOC_LITERAL(6, 60, 12), // "VideoSource*"
QT_MOC_LITERAL(7, 73, 11), // "videoSource"
QT_MOC_LITERAL(8, 85, 12), // "addToHistory"
QT_MOC_LITERAL(9, 98, 4), // "back"
QT_MOC_LITERAL(10, 103, 5), // "pause"
QT_MOC_LITERAL(11, 109, 4), // "stop"
QT_MOC_LITERAL(12, 114, 4), // "skip"
QT_MOC_LITERAL(13, 119, 12), // "skipBackward"
QT_MOC_LITERAL(14, 132, 9), // "skipVideo"
QT_MOC_LITERAL(15, 142, 11), // "openWebPage"
QT_MOC_LITERAL(16, 154, 11), // "copyWebPage"
QT_MOC_LITERAL(17, 166, 13), // "copyVideoLink"
QT_MOC_LITERAL(18, 180, 13), // "openInBrowser"
QT_MOC_LITERAL(19, 194, 15), // "shareViaTwitter"
QT_MOC_LITERAL(20, 210, 16), // "shareViaFacebook"
QT_MOC_LITERAL(21, 227, 13), // "shareViaEmail"
QT_MOC_LITERAL(22, 241, 14), // "removeSelected"
QT_MOC_LITERAL(23, 256, 14), // "moveUpSelected"
QT_MOC_LITERAL(24, 271, 16), // "moveDownSelected"
QT_MOC_LITERAL(25, 288, 16), // "isSidebarVisible"
QT_MOC_LITERAL(26, 305, 20), // "setSidebarVisibility"
QT_MOC_LITERAL(27, 326, 7), // "visible"
QT_MOC_LITERAL(28, 334, 10), // "getSidebar"
QT_MOC_LITERAL(29, 345, 14), // "SidebarWidget*"
QT_MOC_LITERAL(30, 360, 13), // "removeSidebar"
QT_MOC_LITERAL(31, 374, 14), // "restoreSidebar"
QT_MOC_LITERAL(32, 389, 17), // "saveSplitterState"
QT_MOC_LITERAL(33, 407, 13), // "downloadVideo"
QT_MOC_LITERAL(34, 421, 8), // "snapshot"
QT_MOC_LITERAL(35, 430, 10), // "fullscreen"
QT_MOC_LITERAL(36, 441, 14), // "findVideoParts"
QT_MOC_LITERAL(37, 456, 13), // "relatedVideos"
QT_MOC_LITERAL(38, 470, 9), // "canGoBack"
QT_MOC_LITERAL(39, 480, 6), // "goBack"
QT_MOC_LITERAL(40, 487, 12), // "canGoForward"
QT_MOC_LITERAL(41, 500, 9), // "goForward"
QT_MOC_LITERAL(42, 510, 18), // "toggleSubscription"
QT_MOC_LITERAL(43, 529, 16), // "adjustWindowSize"
QT_MOC_LITERAL(44, 546, 24), // "updateSubscriptionAction"
QT_MOC_LITERAL(45, 571, 10), // "subscribed"
QT_MOC_LITERAL(46, 582, 15), // "onItemActivated"
QT_MOC_LITERAL(47, 598, 11), // "QModelIndex"
QT_MOC_LITERAL(48, 610, 5), // "index"
QT_MOC_LITERAL(49, 616, 16), // "selectionChanged"
QT_MOC_LITERAL(50, 633, 14), // "QItemSelection"
QT_MOC_LITERAL(51, 648, 8), // "selected"
QT_MOC_LITERAL(52, 657, 10), // "deselected"
QT_MOC_LITERAL(53, 668, 18), // "activeVideoChanged"
QT_MOC_LITERAL(54, 687, 6), // "Video*"
QT_MOC_LITERAL(55, 694, 5), // "video"
QT_MOC_LITERAL(56, 700, 13), // "previousVideo"
QT_MOC_LITERAL(57, 714, 12), // "selectVideos"
QT_MOC_LITERAL(58, 727, 15), // "QVector<Video*>"
QT_MOC_LITERAL(59, 743, 6), // "videos"
QT_MOC_LITERAL(60, 750, 12), // "gotStreamUrl"
QT_MOC_LITERAL(61, 763, 9), // "streamUrl"
QT_MOC_LITERAL(62, 773, 8), // "audioUrl"
QT_MOC_LITERAL(63, 782, 11), // "handleError"
QT_MOC_LITERAL(64, 794, 7), // "message"
QT_MOC_LITERAL(65, 802, 17), // "mediaStateChanged"
QT_MOC_LITERAL(66, 820, 12), // "Media::State"
QT_MOC_LITERAL(67, 833, 5), // "state"
QT_MOC_LITERAL(68, 839, 15), // "onAboutToFinish"
QT_MOC_LITERAL(69, 855, 18), // "onPlaybackFinished"
QT_MOC_LITERAL(70, 874, 14), // "resumePlayback"
QT_MOC_LITERAL(71, 889, 14), // "onAuthorPushed"
QT_MOC_LITERAL(72, 904, 11), // "searchAgain"
QT_MOC_LITERAL(73, 916, 22) // "resumeWithNewStreamUrl"

    },
    "MediaView\0search\0\0SearchParams*\0"
    "searchParams\0setVideoSource\0VideoSource*\0"
    "videoSource\0addToHistory\0back\0pause\0"
    "stop\0skip\0skipBackward\0skipVideo\0"
    "openWebPage\0copyWebPage\0copyVideoLink\0"
    "openInBrowser\0shareViaTwitter\0"
    "shareViaFacebook\0shareViaEmail\0"
    "removeSelected\0moveUpSelected\0"
    "moveDownSelected\0isSidebarVisible\0"
    "setSidebarVisibility\0visible\0getSidebar\0"
    "SidebarWidget*\0removeSidebar\0"
    "restoreSidebar\0saveSplitterState\0"
    "downloadVideo\0snapshot\0fullscreen\0"
    "findVideoParts\0relatedVideos\0canGoBack\0"
    "goBack\0canGoForward\0goForward\0"
    "toggleSubscription\0adjustWindowSize\0"
    "updateSubscriptionAction\0subscribed\0"
    "onItemActivated\0QModelIndex\0index\0"
    "selectionChanged\0QItemSelection\0"
    "selected\0deselected\0activeVideoChanged\0"
    "Video*\0video\0previousVideo\0selectVideos\0"
    "QVector<Video*>\0videos\0gotStreamUrl\0"
    "streamUrl\0audioUrl\0handleError\0message\0"
    "mediaStateChanged\0Media::State\0state\0"
    "onAboutToFinish\0onPlaybackFinished\0"
    "resumePlayback\0onAuthorPushed\0searchAgain\0"
    "resumeWithNewStreamUrl"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MediaView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      50,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  264,    2, 0x0a /* Public */,
       5,    3,  267,    2, 0x0a /* Public */,
       5,    2,  274,    2, 0x2a /* Public | MethodCloned */,
       5,    1,  279,    2, 0x2a /* Public | MethodCloned */,
      10,    0,  282,    2, 0x0a /* Public */,
      11,    0,  283,    2, 0x0a /* Public */,
      12,    0,  284,    2, 0x0a /* Public */,
      13,    0,  285,    2, 0x0a /* Public */,
      14,    0,  286,    2, 0x0a /* Public */,
      15,    0,  287,    2, 0x0a /* Public */,
      16,    0,  288,    2, 0x0a /* Public */,
      17,    0,  289,    2, 0x0a /* Public */,
      18,    0,  290,    2, 0x0a /* Public */,
      19,    0,  291,    2, 0x0a /* Public */,
      20,    0,  292,    2, 0x0a /* Public */,
      21,    0,  293,    2, 0x0a /* Public */,
      22,    0,  294,    2, 0x0a /* Public */,
      23,    0,  295,    2, 0x0a /* Public */,
      24,    0,  296,    2, 0x0a /* Public */,
      25,    0,  297,    2, 0x0a /* Public */,
      26,    1,  298,    2, 0x0a /* Public */,
      28,    0,  301,    2, 0x0a /* Public */,
      30,    0,  302,    2, 0x0a /* Public */,
      31,    0,  303,    2, 0x0a /* Public */,
      32,    0,  304,    2, 0x0a /* Public */,
      33,    0,  305,    2, 0x0a /* Public */,
      34,    0,  306,    2, 0x0a /* Public */,
      35,    0,  307,    2, 0x0a /* Public */,
      36,    0,  308,    2, 0x0a /* Public */,
      37,    0,  309,    2, 0x0a /* Public */,
      38,    0,  310,    2, 0x0a /* Public */,
      39,    0,  311,    2, 0x0a /* Public */,
      40,    0,  312,    2, 0x0a /* Public */,
      41,    0,  313,    2, 0x0a /* Public */,
      42,    0,  314,    2, 0x0a /* Public */,
      43,    0,  315,    2, 0x0a /* Public */,
      44,    1,  316,    2, 0x0a /* Public */,
      46,    1,  319,    2, 0x08 /* Private */,
      49,    2,  322,    2, 0x08 /* Private */,
      53,    2,  327,    2, 0x08 /* Private */,
      57,    1,  332,    2, 0x08 /* Private */,
      60,    2,  335,    2, 0x08 /* Private */,
      63,    1,  340,    2, 0x08 /* Private */,
      65,    1,  343,    2, 0x08 /* Private */,
      68,    0,  346,    2, 0x08 /* Private */,
      69,    0,  347,    2, 0x08 /* Private */,
      70,    0,  348,    2, 0x08 /* Private */,
      71,    1,  349,    2, 0x08 /* Private */,
      72,    0,  352,    2, 0x08 /* Private */,
      73,    2,  353,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6, QMetaType::Bool, QMetaType::Bool,    7,    8,    9,
    QMetaType::Void, 0x80000000 | 6, QMetaType::Bool,    7,    8,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,   27,
    0x80000000 | 29,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   45,
    QMetaType::Void, 0x80000000 | 47,   48,
    QMetaType::Void, 0x80000000 | 50, 0x80000000 | 50,   51,   52,
    QMetaType::Void, 0x80000000 | 54, 0x80000000 | 54,   55,   56,
    QMetaType::Void, 0x80000000 | 58,   59,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   61,   62,
    QMetaType::Void, QMetaType::QString,   64,
    QMetaType::Void, 0x80000000 | 66,   67,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 47,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   61,   62,

       0        // eod
};

void MediaView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MediaView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->search((*reinterpret_cast< SearchParams*(*)>(_a[1]))); break;
        case 1: _t->setVideoSource((*reinterpret_cast< VideoSource*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 2: _t->setVideoSource((*reinterpret_cast< VideoSource*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->setVideoSource((*reinterpret_cast< VideoSource*(*)>(_a[1]))); break;
        case 4: _t->pause(); break;
        case 5: _t->stop(); break;
        case 6: _t->skip(); break;
        case 7: _t->skipBackward(); break;
        case 8: _t->skipVideo(); break;
        case 9: _t->openWebPage(); break;
        case 10: _t->copyWebPage(); break;
        case 11: _t->copyVideoLink(); break;
        case 12: _t->openInBrowser(); break;
        case 13: _t->shareViaTwitter(); break;
        case 14: _t->shareViaFacebook(); break;
        case 15: _t->shareViaEmail(); break;
        case 16: _t->removeSelected(); break;
        case 17: _t->moveUpSelected(); break;
        case 18: _t->moveDownSelected(); break;
        case 19: { bool _r = _t->isSidebarVisible();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->setSidebarVisibility((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: { SidebarWidget* _r = _t->getSidebar();
            if (_a[0]) *reinterpret_cast< SidebarWidget**>(_a[0]) = std::move(_r); }  break;
        case 22: _t->removeSidebar(); break;
        case 23: _t->restoreSidebar(); break;
        case 24: _t->saveSplitterState(); break;
        case 25: _t->downloadVideo(); break;
        case 26: _t->snapshot(); break;
        case 27: _t->fullscreen(); break;
        case 28: _t->findVideoParts(); break;
        case 29: _t->relatedVideos(); break;
        case 30: { bool _r = _t->canGoBack();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->goBack(); break;
        case 32: { bool _r = _t->canGoForward();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 33: _t->goForward(); break;
        case 34: _t->toggleSubscription(); break;
        case 35: _t->adjustWindowSize(); break;
        case 36: _t->updateSubscriptionAction((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 37: _t->onItemActivated((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 38: _t->selectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 39: _t->activeVideoChanged((*reinterpret_cast< Video*(*)>(_a[1])),(*reinterpret_cast< Video*(*)>(_a[2]))); break;
        case 40: _t->selectVideos((*reinterpret_cast< const QVector<Video*>(*)>(_a[1]))); break;
        case 41: _t->gotStreamUrl((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 42: _t->handleError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 43: _t->mediaStateChanged((*reinterpret_cast< Media::State(*)>(_a[1]))); break;
        case 44: _t->onAboutToFinish(); break;
        case 45: _t->onPlaybackFinished(); break;
        case 46: _t->resumePlayback(); break;
        case 47: _t->onAuthorPushed((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 48: _t->searchAgain(); break;
        case 49: _t->resumeWithNewStreamUrl((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MediaView::staticMetaObject = { {
    QMetaObject::SuperData::link<View::staticMetaObject>(),
    qt_meta_stringdata_MediaView.data,
    qt_meta_data_MediaView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MediaView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MediaView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MediaView.stringdata0))
        return static_cast<void*>(this);
    return View::qt_metacast(_clname);
}

int MediaView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = View::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 50)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 50;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 50)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 50;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
