/****************************************************************************
** Meta object code from reading C++ file 'downloaditem.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/downloaditem.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'downloaditem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DownloadItem_t {
    QByteArrayData data[24];
    char stringdata0[270];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DownloadItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DownloadItem_t qt_meta_stringdata_DownloadItem = {
    {
QT_MOC_LITERAL(0, 0, 12), // "DownloadItem"
QT_MOC_LITERAL(1, 13, 13), // "statusChanged"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 14), // "bufferProgress"
QT_MOC_LITERAL(4, 43, 7), // "percent"
QT_MOC_LITERAL(5, 51, 8), // "progress"
QT_MOC_LITERAL(6, 60, 8), // "finished"
QT_MOC_LITERAL(7, 69, 5), // "error"
QT_MOC_LITERAL(8, 75, 5), // "start"
QT_MOC_LITERAL(9, 81, 4), // "stop"
QT_MOC_LITERAL(10, 86, 8), // "tryAgain"
QT_MOC_LITERAL(11, 95, 4), // "open"
QT_MOC_LITERAL(12, 100, 10), // "openFolder"
QT_MOC_LITERAL(13, 111, 17), // "downloadReadyRead"
QT_MOC_LITERAL(14, 129, 27), // "QNetworkReply::NetworkError"
QT_MOC_LITERAL(15, 157, 4), // "code"
QT_MOC_LITERAL(16, 162, 16), // "downloadProgress"
QT_MOC_LITERAL(17, 179, 13), // "bytesReceived"
QT_MOC_LITERAL(18, 193, 10), // "bytesTotal"
QT_MOC_LITERAL(19, 204, 15), // "metaDataChanged"
QT_MOC_LITERAL(20, 220, 15), // "requestFinished"
QT_MOC_LITERAL(21, 236, 12), // "gotStreamUrl"
QT_MOC_LITERAL(22, 249, 9), // "streamUrl"
QT_MOC_LITERAL(23, 259, 10) // "speedCheck"

    },
    "DownloadItem\0statusChanged\0\0bufferProgress\0"
    "percent\0progress\0finished\0error\0start\0"
    "stop\0tryAgain\0open\0openFolder\0"
    "downloadReadyRead\0QNetworkReply::NetworkError\0"
    "code\0downloadProgress\0bytesReceived\0"
    "bytesTotal\0metaDataChanged\0requestFinished\0"
    "gotStreamUrl\0streamUrl\0speedCheck"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DownloadItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   99,    2, 0x06 /* Public */,
       3,    1,  100,    2, 0x06 /* Public */,
       5,    1,  103,    2, 0x06 /* Public */,
       6,    0,  106,    2, 0x06 /* Public */,
       7,    1,  107,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,  110,    2, 0x0a /* Public */,
       9,    0,  111,    2, 0x0a /* Public */,
      10,    0,  112,    2, 0x0a /* Public */,
      11,    0,  113,    2, 0x0a /* Public */,
      12,    0,  114,    2, 0x0a /* Public */,
      13,    0,  115,    2, 0x08 /* Private */,
       7,    1,  116,    2, 0x08 /* Private */,
      16,    2,  119,    2, 0x08 /* Private */,
      19,    0,  124,    2, 0x08 /* Private */,
      20,    0,  125,    2, 0x08 /* Private */,
      21,    1,  126,    2, 0x08 /* Private */,
      23,    0,  129,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, QMetaType::LongLong, QMetaType::LongLong,   17,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   22,
    QMetaType::Void,

       0        // eod
};

void DownloadItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DownloadItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->statusChanged(); break;
        case 1: _t->bufferProgress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->progress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->finished(); break;
        case 4: _t->error((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 5: _t->start(); break;
        case 6: _t->stop(); break;
        case 7: _t->tryAgain(); break;
        case 8: _t->open(); break;
        case 9: _t->openFolder(); break;
        case 10: _t->downloadReadyRead(); break;
        case 11: _t->error((*reinterpret_cast< QNetworkReply::NetworkError(*)>(_a[1]))); break;
        case 12: _t->downloadProgress((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 13: _t->metaDataChanged(); break;
        case 14: _t->requestFinished(); break;
        case 15: _t->gotStreamUrl((*reinterpret_cast< QUrl(*)>(_a[1]))); break;
        case 16: _t->speedCheck(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNetworkReply::NetworkError >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DownloadItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DownloadItem::statusChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DownloadItem::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DownloadItem::bufferProgress)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DownloadItem::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DownloadItem::progress)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DownloadItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DownloadItem::finished)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (DownloadItem::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DownloadItem::error)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DownloadItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DownloadItem.data,
    qt_meta_data_DownloadItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DownloadItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DownloadItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DownloadItem.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DownloadItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void DownloadItem::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DownloadItem::bufferProgress(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DownloadItem::progress(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DownloadItem::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DownloadItem::error(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
