/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/mainwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[69];
    char stringdata0[861];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 18), // "currentTimeChanged"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 1), // "s"
QT_MOC_LITERAL(4, 33, 11), // "viewChanged"
QT_MOC_LITERAL(5, 45, 8), // "showHome"
QT_MOC_LITERAL(6, 54, 9), // "showMedia"
QT_MOC_LITERAL(7, 64, 13), // "SearchParams*"
QT_MOC_LITERAL(8, 78, 6), // "params"
QT_MOC_LITERAL(9, 85, 12), // "VideoSource*"
QT_MOC_LITERAL(10, 98, 11), // "videoSource"
QT_MOC_LITERAL(11, 110, 15), // "showRegionsView"
QT_MOC_LITERAL(12, 126, 7), // "restore"
QT_MOC_LITERAL(13, 134, 15), // "messageReceived"
QT_MOC_LITERAL(14, 150, 7), // "message"
QT_MOC_LITERAL(15, 158, 4), // "quit"
QT_MOC_LITERAL(16, 163, 18), // "suggestionAccepted"
QT_MOC_LITERAL(17, 182, 11), // "Suggestion*"
QT_MOC_LITERAL(18, 194, 10), // "suggestion"
QT_MOC_LITERAL(19, 205, 6), // "search"
QT_MOC_LITERAL(20, 212, 5), // "query"
QT_MOC_LITERAL(21, 218, 9), // "canGoBack"
QT_MOC_LITERAL(22, 228, 6), // "goBack"
QT_MOC_LITERAL(23, 235, 11), // "showMessage"
QT_MOC_LITERAL(24, 247, 11), // "hideMessage"
QT_MOC_LITERAL(25, 259, 11), // "handleError"
QT_MOC_LITERAL(26, 271, 18), // "isReallyFullScreen"
QT_MOC_LITERAL(27, 290, 9), // "isCompact"
QT_MOC_LITERAL(28, 300, 17), // "missingKeyWarning"
QT_MOC_LITERAL(29, 318, 9), // "visitSite"
QT_MOC_LITERAL(30, 328, 17), // "setDefinitionMode"
QT_MOC_LITERAL(31, 346, 14), // "definitionName"
QT_MOC_LITERAL(32, 361, 8), // "lazyInit"
QT_MOC_LITERAL(33, 370, 6), // "donate"
QT_MOC_LITERAL(34, 377, 11), // "reportIssue"
QT_MOC_LITERAL(35, 389, 5), // "about"
QT_MOC_LITERAL(36, 395, 16), // "toggleFullscreen"
QT_MOC_LITERAL(37, 412, 21), // "updateUIForFullscreen"
QT_MOC_LITERAL(38, 434, 11), // "compactView"
QT_MOC_LITERAL(39, 446, 6), // "enable"
QT_MOC_LITERAL(40, 453, 4), // "stop"
QT_MOC_LITERAL(41, 458, 11), // "searchFocus"
QT_MOC_LITERAL(42, 470, 20), // "toggleDefinitionMode"
QT_MOC_LITERAL(43, 491, 19), // "clearRecentKeywords"
QT_MOC_LITERAL(44, 511, 12), // "stateChanged"
QT_MOC_LITERAL(45, 524, 12), // "Media::State"
QT_MOC_LITERAL(46, 537, 5), // "state"
QT_MOC_LITERAL(47, 543, 4), // "tick"
QT_MOC_LITERAL(48, 548, 4), // "time"
QT_MOC_LITERAL(49, 553, 8), // "volumeUp"
QT_MOC_LITERAL(50, 562, 10), // "volumeDown"
QT_MOC_LITERAL(51, 573, 16), // "toggleVolumeMute"
QT_MOC_LITERAL(52, 590, 13), // "volumeChanged"
QT_MOC_LITERAL(53, 604, 9), // "newVolume"
QT_MOC_LITERAL(54, 614, 18), // "volumeMutedChanged"
QT_MOC_LITERAL(55, 633, 5), // "muted"
QT_MOC_LITERAL(56, 639, 21), // "updateDownloadMessage"
QT_MOC_LITERAL(57, 661, 17), // "downloadsFinished"
QT_MOC_LITERAL(58, 679, 15), // "toggleDownloads"
QT_MOC_LITERAL(59, 695, 4), // "show"
QT_MOC_LITERAL(60, 700, 13), // "setManualPlay"
QT_MOC_LITERAL(61, 714, 7), // "enabled"
QT_MOC_LITERAL(62, 722, 10), // "floatOnTop"
QT_MOC_LITERAL(63, 733, 10), // "showAction"
QT_MOC_LITERAL(64, 744, 28), // "showStopAfterThisInStatusBar"
QT_MOC_LITERAL(65, 773, 16), // "hideFullscreenUI"
QT_MOC_LITERAL(66, 790, 20), // "toggleMenuVisibility"
QT_MOC_LITERAL(67, 811, 31), // "toggleMenuVisibilityWithMessage"
QT_MOC_LITERAL(68, 843, 17) // "toggleToolbarMenu"

    },
    "MainWindow\0currentTimeChanged\0\0s\0"
    "viewChanged\0showHome\0showMedia\0"
    "SearchParams*\0params\0VideoSource*\0"
    "videoSource\0showRegionsView\0restore\0"
    "messageReceived\0message\0quit\0"
    "suggestionAccepted\0Suggestion*\0"
    "suggestion\0search\0query\0canGoBack\0"
    "goBack\0showMessage\0hideMessage\0"
    "handleError\0isReallyFullScreen\0isCompact\0"
    "missingKeyWarning\0visitSite\0"
    "setDefinitionMode\0definitionName\0"
    "lazyInit\0donate\0reportIssue\0about\0"
    "toggleFullscreen\0updateUIForFullscreen\0"
    "compactView\0enable\0stop\0searchFocus\0"
    "toggleDefinitionMode\0clearRecentKeywords\0"
    "stateChanged\0Media::State\0state\0tick\0"
    "time\0volumeUp\0volumeDown\0toggleVolumeMute\0"
    "volumeChanged\0newVolume\0volumeMutedChanged\0"
    "muted\0updateDownloadMessage\0"
    "downloadsFinished\0toggleDownloads\0"
    "show\0setManualPlay\0enabled\0floatOnTop\0"
    "showAction\0showStopAfterThisInStatusBar\0"
    "hideFullscreenUI\0toggleMenuVisibility\0"
    "toggleMenuVisibilityWithMessage\0"
    "toggleToolbarMenu"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      50,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  264,    2, 0x06 /* Public */,
       4,    0,  267,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,  268,    2, 0x0a /* Public */,
       6,    1,  269,    2, 0x0a /* Public */,
       6,    1,  272,    2, 0x0a /* Public */,
      11,    0,  275,    2, 0x0a /* Public */,
      12,    0,  276,    2, 0x0a /* Public */,
      13,    1,  277,    2, 0x0a /* Public */,
      15,    0,  280,    2, 0x0a /* Public */,
      16,    1,  281,    2, 0x0a /* Public */,
      19,    1,  284,    2, 0x0a /* Public */,
      21,    0,  287,    2, 0x0a /* Public */,
      22,    0,  288,    2, 0x0a /* Public */,
      23,    1,  289,    2, 0x0a /* Public */,
      24,    0,  292,    2, 0x0a /* Public */,
      25,    1,  293,    2, 0x0a /* Public */,
      26,    0,  296,    2, 0x0a /* Public */,
      27,    0,  297,    2, 0x0a /* Public */,
      28,    0,  298,    2, 0x0a /* Public */,
      29,    0,  299,    2, 0x0a /* Public */,
      30,    1,  300,    2, 0x0a /* Public */,
      32,    0,  303,    2, 0x08 /* Private */,
      33,    0,  304,    2, 0x08 /* Private */,
      34,    0,  305,    2, 0x08 /* Private */,
      35,    0,  306,    2, 0x08 /* Private */,
      36,    0,  307,    2, 0x08 /* Private */,
      37,    0,  308,    2, 0x08 /* Private */,
      38,    1,  309,    2, 0x08 /* Private */,
      40,    0,  312,    2, 0x08 /* Private */,
      41,    0,  313,    2, 0x08 /* Private */,
      42,    0,  314,    2, 0x08 /* Private */,
      43,    0,  315,    2, 0x08 /* Private */,
      44,    1,  316,    2, 0x08 /* Private */,
      47,    1,  319,    2, 0x08 /* Private */,
      49,    0,  322,    2, 0x08 /* Private */,
      50,    0,  323,    2, 0x08 /* Private */,
      51,    0,  324,    2, 0x08 /* Private */,
      52,    1,  325,    2, 0x08 /* Private */,
      54,    1,  328,    2, 0x08 /* Private */,
      56,    1,  331,    2, 0x08 /* Private */,
      57,    0,  334,    2, 0x08 /* Private */,
      58,    1,  335,    2, 0x08 /* Private */,
      60,    1,  338,    2, 0x08 /* Private */,
      62,    2,  341,    2, 0x08 /* Private */,
      62,    1,  346,    2, 0x28 /* Private | MethodCloned */,
      64,    1,  349,    2, 0x08 /* Private */,
      65,    0,  352,    2, 0x08 /* Private */,
      66,    0,  353,    2, 0x08 /* Private */,
      67,    0,  354,    2, 0x08 /* Private */,
      68,    0,  355,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   31,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   39,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 45,   46,
    QMetaType::Void, QMetaType::LongLong,   48,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QReal,   53,
    QMetaType::Void, QMetaType::Bool,   55,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   59,
    QMetaType::Void, QMetaType::Bool,   61,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,   63,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,   59,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentTimeChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->viewChanged(); break;
        case 2: _t->showHome(); break;
        case 3: _t->showMedia((*reinterpret_cast< SearchParams*(*)>(_a[1]))); break;
        case 4: _t->showMedia((*reinterpret_cast< VideoSource*(*)>(_a[1]))); break;
        case 5: _t->showRegionsView(); break;
        case 6: _t->restore(); break;
        case 7: _t->messageReceived((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->quit(); break;
        case 9: _t->suggestionAccepted((*reinterpret_cast< Suggestion*(*)>(_a[1]))); break;
        case 10: _t->search((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: { bool _r = _t->canGoBack();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->goBack(); break;
        case 13: _t->showMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->hideMessage(); break;
        case 15: _t->handleError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: { bool _r = _t->isReallyFullScreen();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->isCompact();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->missingKeyWarning(); break;
        case 19: _t->visitSite(); break;
        case 20: _t->setDefinitionMode((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 21: _t->lazyInit(); break;
        case 22: _t->donate(); break;
        case 23: _t->reportIssue(); break;
        case 24: _t->about(); break;
        case 25: _t->toggleFullscreen(); break;
        case 26: _t->updateUIForFullscreen(); break;
        case 27: _t->compactView((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->stop(); break;
        case 29: _t->searchFocus(); break;
        case 30: _t->toggleDefinitionMode(); break;
        case 31: _t->clearRecentKeywords(); break;
        case 32: _t->stateChanged((*reinterpret_cast< Media::State(*)>(_a[1]))); break;
        case 33: _t->tick((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 34: _t->volumeUp(); break;
        case 35: _t->volumeDown(); break;
        case 36: _t->toggleVolumeMute(); break;
        case 37: _t->volumeChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 38: _t->volumeMutedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 39: _t->updateDownloadMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 40: _t->downloadsFinished(); break;
        case 41: _t->toggleDownloads((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 42: _t->setManualPlay((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 43: _t->floatOnTop((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 44: _t->floatOnTop((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 45: _t->showStopAfterThisInStatusBar((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 46: _t->hideFullscreenUI(); break;
        case 47: _t->toggleMenuVisibility(); break;
        case 48: _t->toggleMenuVisibilityWithMessage(); break;
        case 49: _t->toggleToolbarMenu(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MainWindow::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::currentTimeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::viewChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 50)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 50;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 50)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 50;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::currentTimeChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void MainWindow::viewChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
