/****************************************************************************
** Meta object code from reading C++ file 'ytchannel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/ytchannel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ytchannel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_YTChannel_t {
    QByteArrayData data[13];
    char stringdata0[145];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_YTChannel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_YTChannel_t qt_meta_stringdata_YTChannel = {
    {
QT_MOC_LITERAL(0, 0, 9), // "YTChannel"
QT_MOC_LITERAL(1, 10, 10), // "infoLoaded"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 15), // "thumbnailLoaded"
QT_MOC_LITERAL(4, 38, 5), // "error"
QT_MOC_LITERAL(5, 44, 7), // "message"
QT_MOC_LITERAL(6, 52, 18), // "notifyCountChanged"
QT_MOC_LITERAL(7, 71, 13), // "updateWatched"
QT_MOC_LITERAL(8, 85, 11), // "unsubscribe"
QT_MOC_LITERAL(9, 97, 13), // "parseResponse"
QT_MOC_LITERAL(10, 111, 5), // "bytes"
QT_MOC_LITERAL(11, 117, 12), // "requestError"
QT_MOC_LITERAL(12, 130, 14) // "storeThumbnail"

    },
    "YTChannel\0infoLoaded\0\0thumbnailLoaded\0"
    "error\0message\0notifyCountChanged\0"
    "updateWatched\0unsubscribe\0parseResponse\0"
    "bytes\0requestError\0storeThumbnail"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_YTChannel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,
       3,    0,   60,    2, 0x06 /* Public */,
       4,    1,   61,    2, 0x06 /* Public */,
       6,    0,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   65,    2, 0x0a /* Public */,
       8,    0,   66,    2, 0x0a /* Public */,
       9,    1,   67,    2, 0x08 /* Private */,
      11,    1,   70,    2, 0x08 /* Private */,
      12,    1,   73,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QByteArray,   10,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QByteArray,   10,

       0        // eod
};

void YTChannel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<YTChannel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->infoLoaded(); break;
        case 1: _t->thumbnailLoaded(); break;
        case 2: _t->error((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->notifyCountChanged(); break;
        case 4: _t->updateWatched(); break;
        case 5: _t->unsubscribe(); break;
        case 6: _t->parseResponse((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 7: _t->requestError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->storeThumbnail((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (YTChannel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YTChannel::infoLoaded)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (YTChannel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YTChannel::thumbnailLoaded)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (YTChannel::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YTChannel::error)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (YTChannel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YTChannel::notifyCountChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject YTChannel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_YTChannel.data,
    qt_meta_data_YTChannel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *YTChannel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *YTChannel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_YTChannel.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int YTChannel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void YTChannel::infoLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void YTChannel::thumbnailLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void YTChannel::error(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void YTChannel::notifyCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
