/****************************************************************************
** Meta object code from reading C++ file 'channelaggregator.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/channelaggregator.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'channelaggregator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ChannelAggregator_t {
    QByteArrayData data[21];
    char stringdata0[246];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ChannelAggregator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ChannelAggregator_t qt_meta_stringdata_ChannelAggregator = {
    {
QT_MOC_LITERAL(0, 0, 17), // "ChannelAggregator"
QT_MOC_LITERAL(1, 18, 14), // "channelChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 10), // "YTChannel*"
QT_MOC_LITERAL(4, 45, 21), // "unwatchedCountChanged"
QT_MOC_LITERAL(5, 67, 5), // "count"
QT_MOC_LITERAL(6, 73, 5), // "start"
QT_MOC_LITERAL(7, 79, 4), // "stop"
QT_MOC_LITERAL(8, 84, 3), // "run"
QT_MOC_LITERAL(9, 88, 20), // "updateUnwatchedCount"
QT_MOC_LITERAL(10, 109, 12), // "videosLoaded"
QT_MOC_LITERAL(11, 122, 15), // "QVector<Video*>"
QT_MOC_LITERAL(12, 138, 6), // "videos"
QT_MOC_LITERAL(13, 145, 18), // "processNextChannel"
QT_MOC_LITERAL(14, 164, 12), // "checkWebPage"
QT_MOC_LITERAL(15, 177, 7), // "channel"
QT_MOC_LITERAL(16, 185, 12), // "parseWebPage"
QT_MOC_LITERAL(17, 198, 5), // "bytes"
QT_MOC_LITERAL(18, 204, 12), // "errorWebPage"
QT_MOC_LITERAL(19, 217, 7), // "message"
QT_MOC_LITERAL(20, 225, 20) // "reallyProcessChannel"

    },
    "ChannelAggregator\0channelChanged\0\0"
    "YTChannel*\0unwatchedCountChanged\0count\0"
    "start\0stop\0run\0updateUnwatchedCount\0"
    "videosLoaded\0QVector<Video*>\0videos\0"
    "processNextChannel\0checkWebPage\0channel\0"
    "parseWebPage\0bytes\0errorWebPage\0message\0"
    "reallyProcessChannel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ChannelAggregator[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       4,    1,   77,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   80,    2, 0x0a /* Public */,
       7,    0,   81,    2, 0x0a /* Public */,
       8,    0,   82,    2, 0x0a /* Public */,
       9,    0,   83,    2, 0x0a /* Public */,
      10,    1,   84,    2, 0x08 /* Private */,
      13,    0,   87,    2, 0x08 /* Private */,
      14,    1,   88,    2, 0x08 /* Private */,
      16,    1,   91,    2, 0x08 /* Private */,
      18,    1,   94,    2, 0x08 /* Private */,
      20,    1,   97,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, QMetaType::Int,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,   15,
    QMetaType::Void, QMetaType::QByteArray,   17,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void, 0x80000000 | 3,   15,

       0        // eod
};

void ChannelAggregator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ChannelAggregator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->channelChanged((*reinterpret_cast< YTChannel*(*)>(_a[1]))); break;
        case 1: _t->unwatchedCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->start(); break;
        case 3: _t->stop(); break;
        case 4: _t->run(); break;
        case 5: _t->updateUnwatchedCount(); break;
        case 6: _t->videosLoaded((*reinterpret_cast< const QVector<Video*>(*)>(_a[1]))); break;
        case 7: _t->processNextChannel(); break;
        case 8: _t->checkWebPage((*reinterpret_cast< YTChannel*(*)>(_a[1]))); break;
        case 9: _t->parseWebPage((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 10: _t->errorWebPage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->reallyProcessChannel((*reinterpret_cast< YTChannel*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ChannelAggregator::*)(YTChannel * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChannelAggregator::channelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ChannelAggregator::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChannelAggregator::unwatchedCountChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ChannelAggregator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ChannelAggregator.data,
    qt_meta_data_ChannelAggregator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ChannelAggregator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ChannelAggregator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ChannelAggregator.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ChannelAggregator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void ChannelAggregator::channelChanged(YTChannel * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ChannelAggregator::unwatchedCountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
