/****************************************************************************
** Meta object code from reading C++ file 'standardfeedsview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/standardfeedsview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'standardfeedsview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_StandardFeedsView_t {
    QByteArrayData data[16];
    char stringdata0[226];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StandardFeedsView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StandardFeedsView_t qt_meta_stringdata_StandardFeedsView = {
    {
QT_MOC_LITERAL(0, 0, 17), // "StandardFeedsView"
QT_MOC_LITERAL(1, 18, 9), // "activated"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 12), // "VideoSource*"
QT_MOC_LITERAL(4, 42, 12), // "standardFeed"
QT_MOC_LITERAL(5, 55, 6), // "appear"
QT_MOC_LITERAL(6, 62, 9), // "disappear"
QT_MOC_LITERAL(7, 72, 4), // "load"
QT_MOC_LITERAL(8, 77, 16), // "layoutCategories"
QT_MOC_LITERAL(9, 94, 19), // "QVector<YTCategory>"
QT_MOC_LITERAL(10, 114, 10), // "categories"
QT_MOC_LITERAL(11, 125, 21), // "selectWorldwideRegion"
QT_MOC_LITERAL(12, 147, 17), // "selectLocalRegion"
QT_MOC_LITERAL(13, 165, 23), // "removeVideoSourceWidget"
QT_MOC_LITERAL(14, 189, 18), // "VideoSourceWidget*"
QT_MOC_LITERAL(15, 208, 17) // "videoSourceWidget"

    },
    "StandardFeedsView\0activated\0\0VideoSource*\0"
    "standardFeed\0appear\0disappear\0load\0"
    "layoutCategories\0QVector<YTCategory>\0"
    "categories\0selectWorldwideRegion\0"
    "selectLocalRegion\0removeVideoSourceWidget\0"
    "VideoSourceWidget*\0videoSourceWidget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StandardFeedsView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   57,    2, 0x0a /* Public */,
       6,    0,   58,    2, 0x0a /* Public */,
       7,    0,   59,    2, 0x0a /* Public */,
       8,    1,   60,    2, 0x08 /* Private */,
      11,    0,   63,    2, 0x08 /* Private */,
      12,    0,   64,    2, 0x08 /* Private */,
      13,    1,   65,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,

       0        // eod
};

void StandardFeedsView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StandardFeedsView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activated((*reinterpret_cast< VideoSource*(*)>(_a[1]))); break;
        case 1: _t->appear(); break;
        case 2: _t->disappear(); break;
        case 3: _t->load(); break;
        case 4: _t->layoutCategories((*reinterpret_cast< const QVector<YTCategory>(*)>(_a[1]))); break;
        case 5: _t->selectWorldwideRegion(); break;
        case 6: _t->selectLocalRegion(); break;
        case 7: _t->removeVideoSourceWidget((*reinterpret_cast< VideoSourceWidget*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StandardFeedsView::*)(VideoSource * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StandardFeedsView::activated)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StandardFeedsView::staticMetaObject = { {
    QMetaObject::SuperData::link<View::staticMetaObject>(),
    qt_meta_stringdata_StandardFeedsView.data,
    qt_meta_data_StandardFeedsView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StandardFeedsView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StandardFeedsView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StandardFeedsView.stringdata0))
        return static_cast<void*>(this);
    return View::qt_metacast(_clname);
}

int StandardFeedsView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = View::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void StandardFeedsView::activated(VideoSource * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
