/****************************************************************************
** Meta object code from reading C++ file 'media.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.16)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../lib/media/src/media.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'media.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.16. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Media_t {
    QByteArrayData data[30];
    char stringdata0[296];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Media_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Media_t qt_meta_stringdata_Media = {
    {
QT_MOC_LITERAL(0, 0, 5), // "Media"
QT_MOC_LITERAL(1, 6, 5), // "error"
QT_MOC_LITERAL(2, 12, 0), // ""
QT_MOC_LITERAL(3, 13, 7), // "message"
QT_MOC_LITERAL(4, 21, 13), // "sourceChanged"
QT_MOC_LITERAL(5, 35, 12), // "bufferStatus"
QT_MOC_LITERAL(6, 48, 5), // "value"
QT_MOC_LITERAL(7, 54, 6), // "loaded"
QT_MOC_LITERAL(8, 61, 7), // "started"
QT_MOC_LITERAL(9, 69, 7), // "stopped"
QT_MOC_LITERAL(10, 77, 6), // "paused"
QT_MOC_LITERAL(11, 84, 1), // "p"
QT_MOC_LITERAL(12, 86, 12), // "stateChanged"
QT_MOC_LITERAL(13, 99, 12), // "Media::State"
QT_MOC_LITERAL(14, 112, 5), // "state"
QT_MOC_LITERAL(15, 118, 15), // "positionChanged"
QT_MOC_LITERAL(16, 134, 2), // "ms"
QT_MOC_LITERAL(17, 137, 13), // "aboutToFinish"
QT_MOC_LITERAL(18, 151, 8), // "finished"
QT_MOC_LITERAL(19, 160, 13), // "volumeChanged"
QT_MOC_LITERAL(20, 174, 18), // "volumeMutedChanged"
QT_MOC_LITERAL(21, 193, 13), // "snapshotReady"
QT_MOC_LITERAL(22, 207, 5), // "image"
QT_MOC_LITERAL(23, 213, 5), // "State"
QT_MOC_LITERAL(24, 219, 12), // "StoppedState"
QT_MOC_LITERAL(25, 232, 12), // "LoadingState"
QT_MOC_LITERAL(26, 245, 14), // "BufferingState"
QT_MOC_LITERAL(27, 260, 12), // "PlayingState"
QT_MOC_LITERAL(28, 273, 11), // "PausedState"
QT_MOC_LITERAL(29, 285, 10) // "ErrorState"

    },
    "Media\0error\0\0message\0sourceChanged\0"
    "bufferStatus\0value\0loaded\0started\0"
    "stopped\0paused\0p\0stateChanged\0"
    "Media::State\0state\0positionChanged\0"
    "ms\0aboutToFinish\0finished\0volumeChanged\0"
    "volumeMutedChanged\0snapshotReady\0image\0"
    "State\0StoppedState\0LoadingState\0"
    "BufferingState\0PlayingState\0PausedState\0"
    "ErrorState"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Media[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       1,  114, // enums/sets
       0,    0, // constructors
       0,       // flags
      14,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x06 /* Public */,
       4,    0,   87,    2, 0x06 /* Public */,
       5,    1,   88,    2, 0x06 /* Public */,
       7,    0,   91,    2, 0x06 /* Public */,
       8,    0,   92,    2, 0x06 /* Public */,
       9,    0,   93,    2, 0x06 /* Public */,
      10,    1,   94,    2, 0x06 /* Public */,
      12,    1,   97,    2, 0x06 /* Public */,
      15,    1,  100,    2, 0x06 /* Public */,
      17,    0,  103,    2, 0x06 /* Public */,
      18,    0,  104,    2, 0x06 /* Public */,
      19,    1,  105,    2, 0x06 /* Public */,
      20,    1,  108,    2, 0x06 /* Public */,
      21,    1,  111,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QReal,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, QMetaType::LongLong,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QReal,    6,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::QImage,   22,

 // enums: name, alias, flags, count, data
      23,   23, 0x0,    6,  119,

 // enum data: key, value
      24, uint(Media::StoppedState),
      25, uint(Media::LoadingState),
      26, uint(Media::BufferingState),
      27, uint(Media::PlayingState),
      28, uint(Media::PausedState),
      29, uint(Media::ErrorState),

       0        // eod
};

void Media::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Media *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->sourceChanged(); break;
        case 2: _t->bufferStatus((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 3: _t->loaded(); break;
        case 4: _t->started(); break;
        case 5: _t->stopped(); break;
        case 6: _t->paused((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->stateChanged((*reinterpret_cast< Media::State(*)>(_a[1]))); break;
        case 8: _t->positionChanged((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 9: _t->aboutToFinish(); break;
        case 10: _t->finished(); break;
        case 11: _t->volumeChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 12: _t->volumeMutedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->snapshotReady((*reinterpret_cast< const QImage(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Media::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::error)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Media::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::sourceChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Media::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::bufferStatus)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Media::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::loaded)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Media::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::started)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Media::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::stopped)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Media::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::paused)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Media::*)(Media::State );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::stateChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Media::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::positionChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Media::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::aboutToFinish)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Media::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::finished)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Media::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::volumeChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Media::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::volumeMutedChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (Media::*)(const QImage & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Media::snapshotReady)) {
                *result = 13;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Media::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Media.data,
    qt_meta_data_Media,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Media::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Media::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Media.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Media::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void Media::error(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Media::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Media::bufferStatus(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Media::loaded()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Media::started()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Media::stopped()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Media::paused(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Media::stateChanged(Media::State _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Media::positionChanged(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void Media::aboutToFinish()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Media::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Media::volumeChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void Media::volumeMutedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void Media::snapshotReady(const QImage & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
